@@ -30,6 +30,12 @@ def construct_query(args) -> Tuple[str, dict]:
         cf.append(" and duration IS NOT NULL " + args.duration)
     if args.size:
         cf.append(" and size IS NOT NULL " + args.size)
+    if args.duration_from_size:
+        cf.append(
+            " and size IS NOT NULL and duration in (select distinct duration from m where 1=1 "
+            + args.duration_from_size
+            + ")"
+        )
 
     cf.extend([" and " + w for w in args.where])
 
@@ -66,14 +72,14 @@ def construct_query(args) -> Tuple[str, dict]:
         construct_search_bindings(args, cf, bindings, m_columns)
 
     if args.table == "media" and not any(
-        [cf, args.where, args.print, args.partial, args.limit != consts.DEFAULT_PLAY_QUEUE]
+        [cf, args.where, args.print, args.partial, args.limit != consts.DEFAULT_PLAY_QUEUE, args.duration_from_size]
     ):
         limit = 60_000
         if args.random:
             limit = consts.DEFAULT_PLAY_QUEUE * 16
 
         if "limit" in args.defaults:
-            cf.append(f"and rowid in (select rowid from media order by random() limit {limit})")
+            cf.append(f"and m.rowid in (select rowid from media order by random() limit {limit})")
     else:
         if args.random:
             args.sort = "random(), " + args.sort
@@ -91,27 +97,34 @@ def construct_query(args) -> Tuple[str, dict]:
         duration = "cast(length(tags) / 4.2 / 220 * 60 as INT) + 10 duration"
 
     cols = args.cols or ["path", "title", duration, "size", "subtitle_count", "is_dir", "rank"]
-    SELECT = "\n,".join([c for c in cols if c in m_columns or c == "*"])
+    SELECT = "\n        , ".join([c for c in cols if c in m_columns or c == "*"])
     LIMIT = "LIMIT " + str(args.limit) if args.limit else ""
     OFFSET = f"OFFSET {args.skip}" if args.skip and args.limit else ""
-    query = f"""SELECT
-        {SELECT}
-    FROM {args.table}
+    query = f"""WITH m as (
+    SELECT rowid, * FROM {args.table}
     WHERE 1=1
-        {args.sql_filter}
         {f'and path like "http%"' if args.safe else ''}
-        {f'and path not like "%{args.keep_dir}%"' if args.post_action == 'askkeep' else ''}
+        {f'and path not like "{Path(args.keep_dir).resolve()}%"' if args.post_action == 'askkeep' else ''}
         {'and time_deleted=0' if 'time_deleted' in m_columns and 'time_deleted' not in args.sql_filter else ''}
         {'AND (score IS NULL OR score > 7)' if 'score' in m_columns else ''}
         {'AND (upvote_ratio IS NULL OR upvote_ratio > 0.73)' if 'upvote_ratio' in m_columns else ''}
+        {'AND time_downloaded = 0' if args.online_media_only else ''}
+        {'AND time_downloaded > 0 AND path not like "http%"' if args.local_media_only else ''}
+    )
+    SELECT
+        {SELECT}
+    FROM m
+    WHERE 1=1
+        {args.sql_filter}
     ORDER BY 1=1
-        {', time_downloaded > 0 desc' if 'time_downloaded' in m_columns and 'time_downloaded' not in args.sql_filter else ''}
         {', video_count > 0 desc' if 'video_count' in m_columns and args.action == SC.watch else ''}
         {', audio_count > 0 desc' if 'audio_count' in m_columns else ''}
+        {', time_downloaded > 0 desc' if 'time_downloaded' in m_columns and 'time_downloaded' not in args.sql_filter else ''}
+        , path like "http%"
         {', width < height desc' if 'width' in m_columns and args.portrait else ''}
         {f', subtitle_count {subtitle_count} desc' if 'subtitle_count' in m_columns and args.action == SC.watch and not any([args.print, consts.PYTEST_RUNNING, 'subtitle_count' in args.where, args.limit != consts.DEFAULT_PLAY_QUEUE]) else ''}
         {', ' + args.sort if args.sort else ''}
-        {', path' if args.print or args.include or args.play_in_order > 0 else ''}
+        , path
         , random()
     {LIMIT} {OFFSET}
     """
@@ -144,16 +157,19 @@ def usage(action, default_db) -> str:
 
     Play media in order (similarly named episodes):
         library {action} --play-in-order
-        There are multiple strictness levels of --play-in-order.
-        If things aren't playing in order try adding more `O`s:
-        library {action} -O    # fast
-        library {action} -OO   # slow, more complex algorithm
-        library {action} -OOO  # slow, ignores most filters
+        There are multiple strictness levels of --play-in-order:
+        library {action} -O   # slow, more complex algorithm
+        library {action} -OO  # above, plus ignores most filters
+        library {action} -OOO # above, plus ignores include/exclude filter during ordinal search
 
     Play recent partially-watched videos (requires mpv history):
         library {action} --partial       # play newest first
         library {action} --partial old   # play oldest first
         library {action} -P o            # equivalent
+        library {action} -P fo           # use file creation time instead of modified time
+
+        library {action} -P p            # sort by progress / duration
+        library {action} -P s            # skip partially watched (only show unseen)
 
     Print instead of play:
         library {action} --print --limit 10  # print the next 10 files
@@ -289,6 +305,15 @@ def usage(action, default_db) -> str:
         library {action} --portrait
         library {action} -w 'width<height' # equivalent
 
+    Constrain media to duration of videos which match any size constraints:
+        library {action} --duration-from-size +700 -u 'duration desc, size desc'
+
+    Constrain media to online-media or local-media:
+        Not to be confused with only local-media which is not "offline" (ie. one HDD disconnected)
+        library {action} --online-media-only
+        library {action} --online-media-only -i  # and ignore playback errors (ie. YouTube video deleted)
+        library {action} --local-media-only
+
     Specify media play order:
         library {action} --sort duration   # play shortest media first
         library {action} -u duration desc  # play longest media first
@@ -318,12 +343,12 @@ def usage(action, default_db) -> str:
 """
 
 
-def parse_size(args):
+def parse_size(size):
     B_TO_MB = 1024 * 1024
     size_mb = 0
     size_rules = ""
 
-    for size_rule in args.size:
+    for size_rule in size:
         if "+" in size_rule:
             # min size rule
             size_rules += f"and size >= {abs(int(size_rule)) * B_TO_MB} "
@@ -338,23 +363,21 @@ def parse_size(args):
 
 
 def parse_duration(args):
-    SEC_TO_M = 60
+    SEC_FROM_M = 60
     duration_m = 0
     duration_rules = ""
 
     for duration_rule in args.duration:
         if "+" in duration_rule:
             # min duration rule
-            duration_rules += f"and duration >= {abs(int(duration_rule)) * SEC_TO_M} "
+            duration_rules += f"and duration >= {int(abs(float(duration_rule)) * SEC_FROM_M)} "
         elif "-" in duration_rule:
             # max duration rule
-            duration_rules += f"and {abs(int(duration_rule)) * SEC_TO_M} >= duration "
+            duration_rules += f"and {int(abs(float(duration_rule)) * SEC_FROM_M)} >= duration "
         else:
             # approximate duration rule
-            duration_m = float(duration_rule) * SEC_TO_M
-            duration_rules += (
-                f"and {duration_m + (duration_m /10)} >= duration and duration >= {duration_m - (duration_m /10)} "
-            )
+            duration_m = float(duration_rule) * SEC_FROM_M
+            duration_rules += f"and {int(duration_m + (duration_m /10))} >= duration and duration >= {int(duration_m - (duration_m /10))} "
 
     return duration_rules
 
@@ -408,6 +431,7 @@ def parse_args(action, default_db, default_chromecast="") -> argparse.Namespace:
 
     parser.add_argument("--duration", "-d", action="append", help=argparse.SUPPRESS)
     parser.add_argument("--size", "-z", action="append", help=argparse.SUPPRESS)
+    parser.add_argument("--duration-from-size", action="append", help=argparse.SUPPRESS)
 
     parser.add_argument("--print", "-p", default=False, const="p", nargs="?", help=argparse.SUPPRESS)
     parser.add_argument("--moved", nargs=2, help=argparse.SUPPRESS)
@@ -438,9 +462,13 @@ def parse_args(action, default_db, default_chromecast="") -> argparse.Namespace:
     parser.add_argument("--keep-cmd", "--keepcmd", help=argparse.SUPPRESS)
     parser.add_argument("--shallow-organize", default="/mnt/d/", help=argparse.SUPPRESS)
 
+    parser.add_argument("--online-media-only", "--online-only", action="store_true", help=argparse.SUPPRESS)
+    parser.add_argument("--local-media-only", "--local-only", action="store_true", help=argparse.SUPPRESS)
+    parser.add_argument("--safe", "-safe", action="store_true", help="Skip generic URLs")
+
+    parser.add_argument("--timeout", "-T", help=argparse.SUPPRESS)
     parser.add_argument("--db", "-db", help=argparse.SUPPRESS)
     parser.add_argument("--ignore-errors", "--ignoreerrors", "-i", action="store_true")
-    parser.add_argument("--safe", "-safe", action="store_true", help="Skip generic URLs")
     parser.add_argument("--verbose", "-v", action="count", default=0)
 
     parser.add_argument(
@@ -485,8 +513,6 @@ def parse_args(action, default_db, default_chromecast="") -> argparse.Namespace:
                     if args.print:
                         args.sort = ["duration", "size"]
 
-    if args.play_in_order > 0:
-        args.sort.append("path")
     args.sort = [override_sort(s) for s in args.sort]
     args.sort = ",".join(args.sort).replace(",,", ",")
 
@@ -497,7 +523,10 @@ def parse_args(action, default_db, default_chromecast="") -> argparse.Namespace:
         args.duration = parse_duration(args)
 
     if args.size:
-        args.size = parse_size(args)
+        args.size = parse_size(args.size)
+
+    if args.duration_from_size:
+        args.duration_from_size = parse_size(args.duration_from_size)
 
     if args.chromecast:
         from catt.api import CattDevice
@@ -510,6 +539,11 @@ def parse_args(action, default_db, default_chromecast="") -> argparse.Namespace:
 
     log.info(utils.dict_filter_bool(args.__dict__))
 
+    if args.keep_dir:
+        args.keep_dir = Path(args.keep_dir).resolve()
+
+    utils.timeout(args.timeout)
+
     args.sock = None
     return args
 
@@ -533,8 +567,8 @@ def chromecast_play(args, m) -> None:
 def is_play_in_order_lvl2(args, media_file) -> bool:
     return any(
         [
-            args.play_in_order >= 2 and args.action != SC.listen,
-            args.play_in_order >= 1 and args.action == SC.listen and "audiobook" in media_file.lower(),
+            args.play_in_order >= 1 and args.action != SC.listen,
+            args.play_in_order >= 0 and args.action == SC.listen and "audiobook" in media_file.lower(),
         ]
     )
 
@@ -593,7 +627,7 @@ def play(args, m: Dict) -> None:
             player.post_act(args, media_file)
 
     elif args.interdimensional_cable:
-        player.socket_play(args, m)
+        return player.socket_play(args, m)
 
     else:
         r = player.local_player(args, m, media_file)
@@ -604,8 +638,8 @@ def play(args, m: Dict) -> None:
             else:
                 raise SystemExit(r.returncode)
 
-        if args.action in (SC.listen, SC.watch):
-            player.post_act(args, media_file)
+    if args.action in (SC.listen, SC.watch):
+        player.post_act(args, media_file)
 
 
 def process_playqueue(args) -> None:
@@ -626,7 +660,7 @@ def process_playqueue(args) -> None:
     if all(
         [
             Path(args.watch_later_directory).exists(),
-            args.play_in_order <= 1,
+            args.play_in_order == 0,
             "sort" in args.defaults,
             not args.partial,
         ]
