@@ -30,27 +30,6 @@ def exit_nicely(_signal, _frame):
 signal.signal(signal.SIGINT, exit_nicely)
 
 
-def no_media_found():
-    print("No media found")
-    raise SystemExit(2)
-
-
-def with_timeout(timeout):
-    def decorator(decorated):
-        @functools.wraps(decorated)
-        def inner(*args, **kwargs):
-            pool = multiprocessing.Pool(1)
-            async_result = pool.apply_async(decorated, args, kwargs)
-            try:
-                return async_result.get(timeout)
-            finally:
-                pool.close()
-
-        return inner
-
-    return decorator
-
-
 def os_bg_kwargs() -> dict:
     # prevent ctrl-c from affecting subprocesses first
 
@@ -125,21 +104,6 @@ def conform(list_: Union[str, Iterable]) -> List:
     return list_
 
 
-def sanitize_url(args, path: str) -> str:
-    matches = consts.REGEX_SUBREDDIT.match(path)
-    if matches:
-        subreddit = conform(matches.groups())[0]
-        frequency = consts.Frequency.Monthly
-        if hasattr(args, "frequency"):
-            frequency = args.frequency
-        return "https://old.reddit.com/r/" + subreddit + "/top/?sort=top&t=" + consts.reddit_frequency(frequency)
-
-    if "/m." in path:
-        return path.replace("/m.", "/www.")
-
-    return path
-
-
 def cmd(*command, strict=True, cwd=None, quiet=True, **kwargs) -> subprocess.CompletedProcess:
     EXP_FILTER = re.compile(
         "|".join(
@@ -180,6 +144,55 @@ def cmd(*command, strict=True, cwd=None, quiet=True, **kwargs) -> subprocess.Com
     return r
 
 
+def timeout(minutes):
+    if minutes and float(minutes) > 0:
+        seconds = int(float(minutes) * 60)
+
+        def exit_timeout(_signal, _frame):
+            print(f"\nReached timeout... ({seconds}s)")
+            cmd("pkill", "mpv")
+            raise SystemExit(124)
+
+        signal.signal(signal.SIGALRM, exit_timeout)
+        signal.alarm(seconds)
+
+
+def no_media_found():
+    print("No media found")
+    raise SystemExit(2)
+
+
+def with_timeout(timeout):
+    def decorator(decorated):
+        @functools.wraps(decorated)
+        def inner(*args, **kwargs):
+            pool = multiprocessing.Pool(1)
+            async_result = pool.apply_async(decorated, args, kwargs)
+            try:
+                return async_result.get(timeout)
+            finally:
+                pool.close()
+
+        return inner
+
+    return decorator
+
+
+def sanitize_url(args, path: str) -> str:
+    matches = consts.REGEX_SUBREDDIT.match(path)
+    if matches:
+        subreddit = conform(matches.groups())[0]
+        frequency = consts.Frequency.Monthly
+        if hasattr(args, "frequency"):
+            frequency = args.frequency
+        return "https://old.reddit.com/r/" + subreddit + "/top/?sort=top&t=" + consts.reddit_frequency(frequency)
+
+    if "/m." in path:
+        return path.replace("/m.", "/www.")
+
+    return path
+
+
 def cmd_detach(*command, **kwargs) -> subprocess.CompletedProcess:
     # https://stackoverflow.com/questions/62521658/python-subprocess-detach-a-process
     # After lb closes, the detached process becomes daemonized (ie. not connected to the terminal so they won't show up in the shell command `jobs`)
@@ -357,33 +370,68 @@ def mpv_enrich(args, media) -> List[dict]:
     return sorted(media, key=lambda m: m.get("time_partial_first") or 0, reverse=True)
 
 
+def mpv_watchlater_value(path, key):
+    data = Path(path).read_text().splitlines()
+    return [s.split("=")[1] for s in data if s.startswith(key)]
+
+
 def mpv_enrich2(args, media) -> List[dict]:
     md5s = {hashlib.md5(m["path"].encode("utf-8")).hexdigest().upper(): m for m in media}
     paths = set(Path(args.watch_later_directory).glob("*"))
 
+    def mpv_watchlater_progress(path):
+        value = mpv_watchlater_value(path, "start")
+        try:
+            return int(float(value[0]))
+        except Exception:
+            return None
+
     previously_watched = [
         {
             **(md5s.get(p.stem) or {}),
             "time_partial_first": int(p.stat().st_ctime),
             "time_partial_last": int(p.stat().st_mtime),
+            "progress": mpv_watchlater_progress(p),
         }
         for p in paths
         if md5s.get(p.stem)
     ]
-    if "s" in args.partial:
+    if "s" in args.partial:  # only unseen
         previously_watched_paths = [m["path"] for m in previously_watched]
         return [m for m in media if m["path"] not in previously_watched_paths]
 
+    def mpv_progress(m):
+        progress = m.get("progress")
+        duration = m.get("duration")
+        if not progress:
+            return 0.0
+        if not duration:
+            return progress / 100  # TODO: idk
+
+        if "w" in args.partial:  # weight by total time
+            return progress / duration * progress
+        else:
+            return progress / duration
+
+    def sorting_hat():
+        if "f" in args.partial:  # first-viewed
+            return lambda m: m.get("time_partial_first") or 0
+        elif "p" in args.partial:  # sort by remaining duration
+            return mpv_progress
+
+        return lambda m: m.get("time_partial_last") or m.get("time_partial_first") or 0
+
     reverse_chronology = True
-    if "o" in args.partial:
+    if "o" in args.partial:  # oldest first
         reverse_chronology = False
 
+    key = sorting_hat()
     if args.print:
         reverse_chronology = not reverse_chronology
 
     media = sorted(
         previously_watched,
-        key=lambda m: m.get("time_partial_last") or m.get("time_partial_first") or 0,
+        key=key,
         reverse=reverse_chronology,
     )
 
